import objectToFormData from 'object-to-formdata';

function CallGroups(core) {
    this.client = core.client;
}

const path = 'dialplan/index.php';
const calledClass = 'classes/dialplan/dialplan.class.php';

const intermediateHandler = (client, options) => {
    return new Promise((resolve, reject) => {
        client(options)
            .then((res) => {
                resolve(res.data);
            })
            .catch((e) => {
                reject(e);
            });
    });
};

function CustomError(reason) {
    Error.call(this, reason);
    this.data = {
        errors: reason
    };
    if (Error.captureStackTrace) {
        Error.captureStackTrace(this, CustomError);
    } else {
        this.stack = new Error().stack;
    }
}
const callGroupMapperFromApi = (item) => {
    const settings = item['queue_parameters'];
    return {
        id: Number(item['cgroup_id']),
        title: String(item['call_groupname']),
        members: item['group_members'].map(item => item),
        settings: {
            queueManager: settings['queue-manager'] || '',
            autoPause: settings['autopause'] || 'no',
            autoPauseTime: settings['autopausetime'],
            autoPauseDelay: settings['autopausedelay'] || '0',
            autoPauseBusy: settings['autopausebusy'] === 'yes',
            autoPauseUnAvail: settings['autopauseunavail'] === 'yes',
            maxLen: settings['maxlen'],
            reportHoldTime: settings['reportholdtime'],
            retry: settings['retry'],
            ringInUse: settings['ringinuse'] === 'yes',
            serviceLevel: settings['servicelevel'],
            setQueueVar: settings['setqueuevar'],
            strategy: settings['strategy'],
            timeout: settings['timeout'],
            weight: settings['weight'],
            wrapUpTime: settings['wrapuptime'],
            rule: settings['context'],
            announce: {
                frequency: settings['announce-frequency'],
                holdTime: settings['announce-holdtime'],
                roundSeconds: settings['announce-round-seconds']
            },
            exitAllMembers: {
                busy: settings['exit_all_members_inuse'] === 'yes',
                paused: settings['exit_all_members_paused'] === 'yes',
                ringing: settings['exit_all_members_ringing'] === 'yes'
            }
        }
    };
};
const callGroupUserMapperFromApi = ([id, value]) => {
    return {
        id, value
    };
};
const callGroupWriteMapper = (id, item) => {
    return objectToFormData({
        action: 'setCallGroup',
        newclass: 'IncomingCalls',
        'cgroup_id': String(id),
        'cgroup_name': String(item.title),
        'cgroup_old_name': '',
        'cgroup_members': item.members.join('|')
    });
};
const callGroupSettingMapper = (id, item) => {
    const settings = {
        'queue-manager': String(item.queueManager || ''),
        'autopause': item.autoPause,
        'autopausetime': item.autoPauseTime,
        'autopausedelay': item.autoPauseDelay,
        'autopausebusy': item.autoPauseBusy ? 'yes' : 'no',
        'autopauseunavail': item.autoPauseUnAvail ? 'yes' : 'no',
        'maxlen': item.maxLen,
        'reportholdtime': item.reportHoldTime,
        'retry': item.retry,
        'ringinuse': item.ringInUse ? 'yes' : 'no',
        'servicelevel': item.serviceLevel,
        'setqueuevar': item.setQueueVar,
        'strategy': item.strategy,
        'timeout': item.timeout,
        'weight': item.weight,
        'wrapuptime': item.wrapUpTime,
        'context': item.rule,
        'announce-holdtime': item.announce.holdTime,
        'announce-frequency': item.announce.frequency,
        'announce-round-seconds': item.announce.roundSeconds,
        'exit_all_members_inuse': item.exitAllMembers.busy ? 'yes' : 'no',
        'exit_all_members_paused': item.exitAllMembers.paused ? 'yes' : 'no',
        'exit_all_members_ringing': item.exitAllMembers.ringing ? 'yes' : 'no'
    };
    return objectToFormData({
        action: 'setCallGroupSettings',
        newclass: 'IncomingCalls',
        'cgroup_id': String(id),
        settings: JSON.stringify({settings})
    });
};

const transormCreateUpdateResponse = function (data) {
    const response = JSON.parse(data);
    if (typeof response.BODY !== 'boolean') {
        const error = {};
        if (response.BODY.edit_cgroup_name) {
            error.title = {
                code: response.BODY.edit_cgroup_name,
                reason: response.BODY.edit_cgroup_name
            };
        } else {
            Object.keys(response.BODY).forEach(item => {
                error[item] = {
                    code: response.BODY[item],
                    reason: response.BODY[item]
                };
            });
        }
        throw new CustomError(error);
    }
    return {
        type: 'result',
        result: {}
    };
};

CustomError.prototype = Object.create(Error.prototype);
CustomError.prototype.constructor = CustomError;

Object.assign(CallGroups.prototype, {
    namespace: 'PBX.dialplans.callGroups',
    getList() {
        return intermediateHandler(this.client._transport, {
            url: path,
            method: 'GET',
            params: {
                'class': calledClass,
                'action': 'getAllCallGroups',
                'newclass': 'IncomingCalls'
            },
            transformResponse: [
                function(data) {
                    const response = JSON.parse(data);
                    if (response.error) {
                        throw new CustomError(response.error);
                    }
                    return {
                        type: 'result',
                        result: {
                            records: response.BODY.map(item => callGroupMapperFromApi(item))
                        }
                    };
                }
            ]
        });
    },
    getUsers(str = '') {
        return intermediateHandler(this.client._transport, {
            url: path,
            method: 'GET',
            params: {
                'action': 'getUsersWithF',
                'text': str
            },
            transformResponse: [
                function(data) {
                    const response = JSON.parse(data);
                    if (response.error) {
                        throw new CustomError(response.error);
                    }
                    let total = response.count;
                    delete response.count;
                    return {
                        type: 'result',
                        result: {
                            records: Object.entries(response).map(item => callGroupUserMapperFromApi(item)),
                            total: total
                        }
                    };
                }
            ]
        });
    },
    fetchUsers(items) {
        return intermediateHandler(this.client._transport, {
            url: path,
            method: 'GET',
            params: {
                'action': 'getMembersByIdF',
                'vals': items
            },
            transformResponse: [
                function(data) {
                    const response = JSON.parse(data);
                    if (response.error) {
                        throw new CustomError(response.error);
                    }
                    let total = response.count;
                    delete response.count;
                    return {
                        type: 'result',
                        result: {
                            records: Object.entries(response).map(item => callGroupUserMapperFromApi(item)),
                            total: total
                        }
                    };
                }
            ]
        });
    },
    create(params) {
        return intermediateHandler(this.client._transport, {
            url: path,
            method: 'POST',
            params: {
                'class': calledClass
            },
            data: callGroupWriteMapper('new', params),
            transformResponse: [transormCreateUpdateResponse]
        });
    },
    update(id, params) {
        return intermediateHandler(this.client._transport, {
            url: path,
            method: 'POST',
            params: {
                'class': calledClass
            },
            data: callGroupWriteMapper(id, params),
            transformResponse: [transormCreateUpdateResponse]
        });
    },
    remove({id}) {
        return intermediateHandler(this.client._transport, {
            url: path,
            method: 'GET',
            params: {
                'class': calledClass,
                'action': 'removeCallgroups',
                'newclass': 'IncomingCalls',
                'entries': id
            },
            transformResponse: [
                function (data) {
                    const response = JSON.parse(data);
                    if (response.error) {
                        throw new CustomError(response.error);
                    }
                    return {
                        type: 'result',
                        result: {}
                    };
                }
            ]
        });
    },
    updateSettings(id, params) {
        return intermediateHandler(this.client._transport, {
            url: path,
            method: 'POST',
            params: {
                'class': calledClass
            },
            data: callGroupSettingMapper(id, params),
            transformResponse: [
                function(data) {
                    const response = JSON.parse(data);
                    if (typeof response.BODY !== 'boolean') {
                        throw new CustomError({
                            title: {
                                code: response.BODY.edit_cgroup_name,
                                reason: response.BODY.edit_cgroup_name
                            }});
                    }
                    return {
                        type: 'result',
                        result: {}
                    };
                }]
        });
    }
});

export default CallGroups;
